use chrono::{Duration, Utc};
use jsonwebtoken::{decode, encode, DecodingKey, EncodingKey, Header, Validation};
use serde::{Deserialize, Serialize};

use crate::error::{AppError, AppResult};

#[derive(Debug, Serialize, Deserialize)]
pub struct Claims {
    pub did: String,
    pub handle: String,
    pub exp: usize,
}

pub struct JwtService {
    encoding_key: EncodingKey,
    decoding_key: DecodingKey,
}

impl JwtService {
    pub fn new(secret: &str) -> Self {
        Self {
            encoding_key: EncodingKey::from_secret(secret.as_ref()),
            decoding_key: DecodingKey::from_secret(secret.as_ref()),
        }
    }

    pub fn create_token(&self, did: &str, handle: &str, expires_in_minutes: u64) -> AppResult<String> {
        let expiration = Utc::now()
            .checked_add_signed(Duration::minutes(expires_in_minutes as i64))
            .ok_or_else(|| AppError::internal("Failed to calculate expiration time"))?
            .timestamp() as usize;

        let claims = Claims {
            did: did.to_string(),
            handle: handle.to_string(),
            exp: expiration,
        };

        encode(&Header::default(), &claims, &self.encoding_key)
            .map_err(AppError::Jwt)
    }

    pub fn verify_token(&self, token: &str) -> AppResult<Claims> {
        let token_data = decode::<Claims>(token, &self.decoding_key, &Validation::default())
            .map_err(AppError::Jwt)?;

        Ok(token_data.claims)
    }
}

/// Mock atproto authentication service
/// In a real implementation, this would integrate with actual atproto services
pub struct AtprotoAuthService {
    jwt_service: JwtService,
}

impl AtprotoAuthService {
    pub fn new(secret: &str) -> Self {
        Self {
            jwt_service: JwtService::new(secret),
        }
    }

    /// Authenticate user with atproto credentials
    /// This is a mock implementation - in reality would validate against atproto PDS
    pub async fn authenticate(&self, identifier: &str, _password: &str) -> AppResult<AuthenticatedUser> {
        // Mock validation - in real implementation:
        // 1. Connect to user's PDS
        // 2. Verify credentials
        // 3. Get user DID and handle
        
        // For now, treat identifier as DID or handle
        let (did, handle) = if identifier.starts_with("did:") {
            (identifier.to_string(), extract_handle_from_did(identifier))
        } else {
            (format!("did:plc:{}", generate_mock_plc_id()), identifier.to_string())
        };

        Ok(AuthenticatedUser { did, handle })
    }

    pub fn create_access_token(&self, user: &AuthenticatedUser, expires_in_minutes: u64) -> AppResult<String> {
        self.jwt_service.create_token(&user.did, &user.handle, expires_in_minutes)
    }

    pub fn verify_access_token(&self, token: &str) -> AppResult<Claims> {
        self.jwt_service.verify_token(token)
    }
}

#[derive(Debug, Clone)]
pub struct AuthenticatedUser {
    pub did: String,
    pub handle: String,
}

/// Extract handle from DID (mock implementation)
fn extract_handle_from_did(did: &str) -> String {
    // In a real implementation, this would resolve the DID to get the handle
    // For now, use a simple mock
    did.split(':').last().unwrap_or("unknown").to_string()
}

/// Generate mock PLC identifier
fn generate_mock_plc_id() -> String {
    use uuid::Uuid;
    Uuid::new_v4().to_string().replace('-', "")[..24].to_string()
}