use sqlx::{Pool, Postgres, Sqlite, Row};
use sqlx::migrate::MigrateDatabase;
use crate::error::{AppError, AppResult};
use std::str::FromStr;

#[derive(Clone)]
pub enum Database {
    Postgres(Pool<Postgres>),
    Sqlite(Pool<Sqlite>),
}

impl Database {
    pub async fn connect(database_url: &str) -> AppResult<Self> {
        if database_url.starts_with("postgres://") || database_url.starts_with("postgresql://") {
            let pool = sqlx::postgres::PgPoolOptions::new()
                .max_connections(10)
                .connect(database_url)
                .await
                .map_err(AppError::Database)?;
            Ok(Database::Postgres(pool))
        } else if database_url.starts_with("sqlite://") {
            // Extract the path from sqlite:// URL
            let db_path = database_url.trim_start_matches("sqlite://");
            
            // Create the database file if it doesn't exist
            if !Sqlite::database_exists(database_url).await.unwrap_or(false) {
                Sqlite::create_database(database_url)
                    .await
                    .map_err(AppError::Database)?;
            }
            
            let pool = sqlx::sqlite::SqlitePoolOptions::new()
                .max_connections(5)
                .connect(database_url)
                .await
                .map_err(AppError::Database)?;
                
            Ok(Database::Sqlite(pool))
        } else {
            Err(AppError::Configuration(format!(
                "Unsupported database URL: {}",
                database_url
            )))
        }
    }

    pub async fn migrate(&self) -> AppResult<()> {
        match self {
            Database::Postgres(pool) => {
                sqlx::migrate!("./migrations/postgres")
                    .run(pool)
                    .await
                    .map_err(AppError::Migration)?;
            }
            Database::Sqlite(pool) => {
                sqlx::migrate!("./migrations/sqlite")
                    .run(pool)
                    .await
                    .map_err(AppError::Migration)?;
            }
        }
        Ok(())
    }


    /// Get a generic connection for complex operations
    pub async fn acquire(&self) -> AppResult<DatabaseConnection> {
        match self {
            Database::Postgres(pool) => {
                let conn = pool.acquire().await.map_err(AppError::Database)?;
                Ok(DatabaseConnection::Postgres(conn))
            }
            Database::Sqlite(pool) => {
                let conn = pool.acquire().await.map_err(AppError::Database)?;
                Ok(DatabaseConnection::Sqlite(conn))
            }
        }
    }

    /// Begin a transaction
    pub async fn begin(&self) -> AppResult<DatabaseTransaction> {
        match self {
            Database::Postgres(pool) => {
                let tx = pool.begin().await.map_err(AppError::Database)?;
                Ok(DatabaseTransaction::Postgres(tx))
            }
            Database::Sqlite(pool) => {
                let tx = pool.begin().await.map_err(AppError::Database)?;
                Ok(DatabaseTransaction::Sqlite(tx))
            }
        }
    }
}

pub enum DatabaseConnection {
    Postgres(sqlx::pool::PoolConnection<Postgres>),
    Sqlite(sqlx::pool::PoolConnection<Sqlite>),
}

pub enum DatabaseTransaction {
    Postgres(sqlx::Transaction<'static, Postgres>),
    Sqlite(sqlx::Transaction<'static, Sqlite>),
}

impl DatabaseTransaction {
    pub async fn commit(self) -> AppResult<()> {
        match self {
            DatabaseTransaction::Postgres(tx) => {
                tx.commit().await.map_err(AppError::Database)?;
            }
            DatabaseTransaction::Sqlite(tx) => {
                tx.commit().await.map_err(AppError::Database)?;
            }
        }
        Ok(())
    }

    pub async fn rollback(self) -> AppResult<()> {
        match self {
            DatabaseTransaction::Postgres(tx) => {
                tx.rollback().await.map_err(AppError::Database)?;
            }
            DatabaseTransaction::Sqlite(tx) => {
                tx.rollback().await.map_err(AppError::Database)?;
            }
        }
        Ok(())
    }
}

// Macros for database-agnostic queries
#[macro_export]
macro_rules! query_as {
    ($struct:ty, $query:expr, $db:expr) => {
        match $db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_all(pool)
                    .await
                    .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_all(pool)
                    .await
                    .map_err(AppError::Database)
            }
        }
    };
}

#[macro_export]
macro_rules! query_one_as {
    ($struct:ty, $query:expr, $db:expr) => {
        match $db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)
            }
        }
    };
}

#[macro_export]
macro_rules! query_optional_as {
    ($struct:ty, $query:expr, $db:expr) => {
        match $db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_optional(pool)
                    .await
                    .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, $struct>($query)
                    .fetch_optional(pool)
                    .await
                    .map_err(AppError::Database)
            }
        }
    };
}