use crate::{
    error::{AppError, AppResult},
    models::*,
};
use reqwest::Client;
use serde_json::json;

pub struct AtprotoService {
    client: Client,
    session: Option<String>,
}

impl AtprotoService {
    pub fn new() -> Self {
        Self {
            client: Client::new(),
            session: None,
        }
    }

    pub fn with_session(session: String) -> Self {
        Self {
            client: Client::new(),
            session: Some(session),
        }
    }

    /// Create a card record in user's atproto PDS
    pub async fn create_card_record(
        &self,
        did: &str,
        card: &UserCard,
        master: &CardMaster,
    ) -> AppResult<String> {
        let session = self.session.as_ref()
            .ok_or_else(|| AppError::authentication("No atproto session available"))?;

        let record_data = AtprotoCardRecord {
            record_type: "ai.card.collection".to_string(),
            card_id: card.card_id,
            cp: card.cp,
            status: card.status.as_str().to_string(),
            obtained_at: card.obtained_at,
            is_unique: card.is_unique,
            unique_id: card.unique_id,
        };

        // Determine PDS endpoint from DID
        let pds_url = self.resolve_pds_from_did(did).await?;

        let response = self
            .client
            .post(&format!("{}/xrpc/com.atproto.repo.createRecord", pds_url))
            .header("Authorization", format!("Bearer {}", session))
            .json(&json!({
                "repo": did,
                "collection": "ai.card.collection",
                "record": record_data
            }))
            .send()
            .await
            .map_err(AppError::HttpClient)?;

        if !response.status().is_success() {
            return Err(AppError::ExternalService(format!(
                "Failed to create atproto record: HTTP {}",
                response.status()
            )));
        }

        let result: serde_json::Value = response
            .json()
            .await
            .map_err(AppError::HttpClient)?;

        let uri = result["uri"]
            .as_str()
            .ok_or_else(|| AppError::ExternalService("No URI in response".to_string()))?;

        Ok(uri.to_string())
    }

    /// List card records from user's PDS
    pub async fn list_card_records(&self, did: &str) -> AppResult<Vec<serde_json::Value>> {
        let session = self.session.as_ref()
            .ok_or_else(|| AppError::authentication("No atproto session available"))?;

        let pds_url = self.resolve_pds_from_did(did).await?;

        let response = self
            .client
            .get(&format!("{}/xrpc/com.atproto.repo.listRecords", pds_url))
            .header("Authorization", format!("Bearer {}", session))
            .query(&[
                ("repo", did),
                ("collection", "ai.card.collection"),
            ])
            .send()
            .await
            .map_err(AppError::HttpClient)?;

        if !response.status().is_success() {
            return Err(AppError::ExternalService(format!(
                "Failed to list atproto records: HTTP {}",
                response.status()
            )));
        }

        let result: serde_json::Value = response
            .json()
            .await
            .map_err(AppError::HttpClient)?;

        let records = result["records"]
            .as_array()
            .ok_or_else(|| AppError::ExternalService("No records in response".to_string()))?;

        Ok(records.clone())
    }

    /// Resolve PDS endpoint from DID
    async fn resolve_pds_from_did(&self, did: &str) -> AppResult<String> {
        // This is a simplified resolution
        // In a real implementation, you would:
        // 1. Parse the DID to get the method and identifier
        // 2. Query the appropriate resolver (PLC directory, etc.)
        // 3. Get the serviceEndpoint for the PDS

        if did.starts_with("did:plc:") {
            // For PLC DIDs, query the PLC directory
            let plc_id = did.strip_prefix("did:plc:").unwrap();
            self.resolve_plc_did(plc_id).await
        } else if did.starts_with("did:web:") {
            // For web DIDs, construct URL from domain
            let domain = did.strip_prefix("did:web:").unwrap();
            Ok(format!("https://{}", domain))
        } else {
            // Fallback to Bluesky PDS
            Ok("https://bsky.social".to_string())
        }
    }

    /// Resolve PLC DID to PDS endpoint
    async fn resolve_plc_did(&self, plc_id: &str) -> AppResult<String> {
        let response = self
            .client
            .get(&format!("https://plc.directory/{}", plc_id))
            .send()
            .await
            .map_err(AppError::HttpClient)?;

        if !response.status().is_success() {
            return Ok("https://bsky.social".to_string()); // Fallback
        }

        let did_doc: serde_json::Value = response
            .json()
            .await
            .map_err(AppError::HttpClient)?;

        // Extract PDS endpoint from DID document
        if let Some(services) = did_doc["service"].as_array() {
            for service in services {
                if service["id"] == "#atproto_pds" {
                    if let Some(endpoint) = service["serviceEndpoint"].as_str() {
                        return Ok(endpoint.to_string());
                    }
                }
            }
        }

        // Fallback to Bluesky
        Ok("https://bsky.social".to_string())
    }

    /// Authenticate with atproto and get session
    pub async fn authenticate(&self, identifier: &str, password: &str) -> AppResult<(String, String)> {
        // Try multiple PDS endpoints for authentication
        let pds_endpoints = [
            "https://bsky.social",
            "https://staging.bsky.app",
            // Add more PDS endpoints as needed
        ];

        for pds_url in pds_endpoints {
            match self.try_authenticate_at_pds(pds_url, identifier, password).await {
                Ok((session, did)) => return Ok((session, did)),
                Err(_) => continue, // Try next PDS
            }
        }

        Err(AppError::authentication("Failed to authenticate with any PDS"))
    }

    /// Try authentication at a specific PDS
    async fn try_authenticate_at_pds(
        &self,
        pds_url: &str,
        identifier: &str,
        password: &str,
    ) -> AppResult<(String, String)> {
        let response = self
            .client
            .post(&format!("{}/xrpc/com.atproto.server.createSession", pds_url))
            .json(&json!({
                "identifier": identifier,
                "password": password
            }))
            .send()
            .await
            .map_err(AppError::HttpClient)?;

        if !response.status().is_success() {
            return Err(AppError::authentication("Invalid credentials"));
        }

        let result: serde_json::Value = response
            .json()
            .await
            .map_err(AppError::HttpClient)?;

        let access_jwt = result["accessJwt"]
            .as_str()
            .ok_or_else(|| AppError::authentication("No access token in response"))?;

        let did = result["did"]
            .as_str()
            .ok_or_else(|| AppError::authentication("No DID in response"))?;

        Ok((access_jwt.to_string(), did.to_string()))
    }
}