use crate::{
    error::{AppError, AppResult},
    models::*,
};
use reqwest::Client;
use std::collections::HashMap;

pub struct CardMasterService {
    client: Client,
    master_url: String,
}

impl CardMasterService {
    pub fn new(master_url: String) -> Self {
        Self {
            client: Client::new(),
            master_url,
        }
    }

    /// Fetch card master data from external source (ai.json)
    pub async fn fetch_external_card_data(&self) -> AppResult<Vec<ExternalCard>> {
        let response = self
            .client
            .get(&self.master_url)
            .timeout(std::time::Duration::from_secs(10))
            .send()
            .await
            .map_err(AppError::HttpClient)?;

        if !response.status().is_success() {
            return Err(AppError::ExternalService(format!(
                "Failed to fetch card data: HTTP {}",
                response.status()
            )));
        }

        let data: ExternalCardData = response
            .json()
            .await
            .map_err(AppError::HttpClient)?;

        Ok(data.ai.card.cards)
    }

    /// Get fallback card data if external fetch fails
    pub fn get_fallback_card_data(&self) -> Vec<ExternalCard> {
        vec![
            ExternalCard {
                id: 0,
                name: "ai".to_string(),
                cp: CpRange { min: 100, max: 200 },
                color: "#4A90E2".to_string(),
                skill: "Core existence essence".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 1,
                name: "dream".to_string(),
                cp: CpRange { min: 90, max: 180 },
                color: "#9B59B6".to_string(),
                skill: "Vision manifestation".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 2,
                name: "radiance".to_string(),
                cp: CpRange { min: 110, max: 220 },
                color: "#F39C12".to_string(),
                skill: "Brilliant energy".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 3,
                name: "neutron".to_string(),
                cp: CpRange { min: 120, max: 240 },
                color: "#34495E".to_string(),
                skill: "Dense core power".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 4,
                name: "sun".to_string(),
                cp: CpRange { min: 130, max: 260 },
                color: "#E74C3C".to_string(),
                skill: "Solar radiance".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 5,
                name: "night".to_string(),
                cp: CpRange { min: 80, max: 160 },
                color: "#2C3E50".to_string(),
                skill: "Shadow stealth".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 6,
                name: "snow".to_string(),
                cp: CpRange { min: 70, max: 140 },
                color: "#ECF0F1".to_string(),
                skill: "Crystal freeze".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 7,
                name: "thunder".to_string(),
                cp: CpRange { min: 140, max: 280 },
                color: "#F1C40F".to_string(),
                skill: "Electric storm".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 8,
                name: "ultimate".to_string(),
                cp: CpRange { min: 150, max: 300 },
                color: "#8E44AD".to_string(),
                skill: "Maximum form".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 9,
                name: "sword".to_string(),
                cp: CpRange { min: 160, max: 320 },
                color: "#95A5A6".to_string(),
                skill: "Truth cutting".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 10,
                name: "destruction".to_string(),
                cp: CpRange { min: 170, max: 340 },
                color: "#C0392B".to_string(),
                skill: "Entropy force".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 11,
                name: "earth".to_string(),
                cp: CpRange { min: 90, max: 180 },
                color: "#27AE60".to_string(),
                skill: "Ground foundation".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 12,
                name: "galaxy".to_string(),
                cp: CpRange { min: 180, max: 360 },
                color: "#3498DB".to_string(),
                skill: "Cosmic expanse".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 13,
                name: "create".to_string(),
                cp: CpRange { min: 100, max: 200 },
                color: "#16A085".to_string(),
                skill: "Generation power".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 14,
                name: "supernova".to_string(),
                cp: CpRange { min: 200, max: 400 },
                color: "#E67E22".to_string(),
                skill: "Stellar explosion".to_string(),
                lang: None,
            },
            ExternalCard {
                id: 15,
                name: "world".to_string(),
                cp: CpRange { min: 250, max: 500 },
                color: "#9B59B6".to_string(),
                skill: "Reality control".to_string(),
                lang: None,
            },
        ]
    }

    /// Get card master data, trying external source first then fallback
    pub async fn get_card_master_data(&self) -> Vec<ExternalCard> {
        match self.fetch_external_card_data().await {
            Ok(cards) => {
                tracing::info!("Fetched {} cards from external source", cards.len());
                cards
            }
            Err(e) => {
                tracing::warn!("Failed to fetch external card data: {}, using fallback", e);
                self.get_fallback_card_data()
            }
        }
    }

    /// Convert external card data to database format
    pub fn external_to_card_master(external: &ExternalCard) -> CardMaster {
        let description = if let Some(lang) = &external.lang {
            if let Some(ja) = &lang.ja {
                if let Some(name) = &ja.name {
                    format!("{} - {}", name, external.skill)
                } else {
                    external.skill.clone()
                }
            } else {
                external.skill.clone()
            }
        } else {
            external.skill.clone()
        };

        CardMaster {
            id: external.id,
            name: external.name.clone(),
            base_cp_min: external.cp.min,
            base_cp_max: external.cp.max,
            color: external.color.clone(),
            description,
        }
    }
}