use crate::{
    config::GachaConfig,
    database::{Database, DatabaseTransaction},
    error::{AppError, AppResult},
    models::*,
    query_as, query_one_as, query_optional_as,
    services::CardMasterService,
};
use chrono::Utc;
use rand::Rng;
use std::collections::HashMap;
use uuid::Uuid;

pub struct GachaService {
    config: GachaConfig,
}

impl GachaService {
    pub fn new(config: GachaConfig) -> Self {
        Self { config }
    }

    /// Main gacha draw function
    pub async fn draw_card(
        &self,
        db: &Database,
        user_did: &str,
        is_paid: bool,
        pool_id: Option<i32>,
    ) -> AppResult<CardDrawResponse> {
        let mut tx = db.begin().await?;

        // Get or create user
        let user = self.get_or_create_user(&mut tx, user_did).await?;

        // Determine card rarity
        let rarity = self.determine_rarity(is_paid, pool_id)?;

        // Select a card based on rarity and pool
        let card_master = self.select_card_master(&mut tx, &rarity, pool_id).await?;

        // Calculate CP based on rarity
        let cp = self.calculate_cp(&card_master, &rarity);

        // Check if this will be a unique card
        let is_unique = rarity == CardRarity::Unique;
        
        // For unique cards, check availability
        if is_unique {
            if let Some(_existing) = self.check_unique_card_availability(&mut tx, card_master.id).await? {
                // Unique card already taken, fallback to Kira
                return self.draw_card_with_fallback(&mut tx, user.id, &card_master, CardRarity::Kira, is_paid).await;
            }
        }

        // Create the user card
        let user_card = self.create_user_card(
            &mut tx,
            user.id,
            &card_master,
            cp,
            &rarity,
            is_unique,
        ).await?;

        // Record draw history
        let draw_history = self.record_draw_history(
            &mut tx,
            user.id,
            card_master.id,
            &rarity,
            cp,
            is_paid,
        ).await?;

        // Register unique card if applicable
        if is_unique {
            self.register_unique_card(&mut tx, &user_card, user_did).await?;
        }

        tx.commit().await?;

        Ok(CardDrawResponse {
            card: UserCardResponse {
                id: user_card.id,
                card_id: user_card.card_id,
                cp: user_card.cp,
                status: user_card.status,
                skill: user_card.skill,
                obtained_at: user_card.obtained_at,
                is_unique: user_card.is_unique,
                unique_id: user_card.unique_id,
            },
            master: CardMasterResponse {
                id: card_master.id,
                name: card_master.name,
                base_cp_min: card_master.base_cp_min,
                base_cp_max: card_master.base_cp_max,
                color: card_master.color,
                description: card_master.description,
            },
            is_unique,
            animation_type: self.get_animation_type(&rarity),
            draw_history_id: draw_history.id,
        })
    }

    /// Determine card rarity based on probabilities
    fn determine_rarity(&self, is_paid: bool, _pool_id: Option<i32>) -> AppResult<CardRarity> {
        let mut rng = rand::thread_rng();
        let rand_val: f64 = rng.gen_range(0.0..100.0);

        let cumulative_probs = self.config.cumulative_probabilities(is_paid);
        let mut cumulative = 0.0;

        for (prob, rarity) in cumulative_probs {
            cumulative += prob;
            if rand_val < cumulative {
                return Ok(rarity);
            }
        }

        // Fallback to normal if no match (should never happen)
        Ok(CardRarity::Normal)
    }

    /// Select a card master based on rarity and pool
    async fn select_card_master(
        &self,
        tx: &mut DatabaseTransaction,
        rarity: &CardRarity,
        _pool_id: Option<i32>,
    ) -> AppResult<CardMaster> {
        // For now, randomly select from all available cards
        // In a full implementation, this would consider pool restrictions
        let cards = match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, CardMaster>("SELECT * FROM card_master ORDER BY RANDOM() LIMIT 1")
                    .fetch_one(&mut **tx)
                    .await
                    .map_err(AppError::Database)?
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, CardMaster>("SELECT * FROM card_master ORDER BY RANDOM() LIMIT 1")
                    .fetch_one(&mut **tx)
                    .await
                    .map_err(AppError::Database)?
            }
        };

        Ok(cards)
    }

    /// Calculate CP based on base CP and rarity multiplier
    fn calculate_cp(&self, card_master: &CardMaster, rarity: &CardRarity) -> i32 {
        let mut rng = rand::thread_rng();
        let base_cp = rng.gen_range(card_master.base_cp_min..=card_master.base_cp_max);
        let multiplier = rarity.multiplier();
        (base_cp as f64 * multiplier) as i32
    }

    /// Check if a unique card is available
    async fn check_unique_card_availability(
        &self,
        tx: &mut DatabaseTransaction,
        card_id: i32,
    ) -> AppResult<Option<UniqueCardRegistry>> {
        match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, UniqueCardRegistry>(
                    "SELECT * FROM unique_card_registry WHERE card_id = $1"
                )
                .bind(card_id)
                .fetch_optional(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, UniqueCardRegistry>(
                    "SELECT * FROM unique_card_registry WHERE card_id = ?"
                )
                .bind(card_id)
                .fetch_optional(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    /// Create a user card
    async fn create_user_card(
        &self,
        tx: &mut DatabaseTransaction,
        user_id: i32,
        card_master: &CardMaster,
        cp: i32,
        rarity: &CardRarity,
        is_unique: bool,
    ) -> AppResult<UserCard> {
        let unique_id = if is_unique { Some(Uuid::new_v4()) } else { None };
        let now = Utc::now();

        match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, UserCard>(
                    r#"
                    INSERT INTO user_cards (user_id, card_id, cp, status, obtained_at, is_unique, unique_id)
                    VALUES ($1, $2, $3, $4, $5, $6, $7)
                    RETURNING *
                    "#
                )
                .bind(user_id)
                .bind(card_master.id)
                .bind(cp)
                .bind(rarity)
                .bind(now)
                .bind(is_unique)
                .bind(unique_id)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, UserCard>(
                    r#"
                    INSERT INTO user_cards (user_id, card_id, cp, status, obtained_at, is_unique, unique_id)
                    VALUES (?, ?, ?, ?, ?, ?, ?)
                    RETURNING *
                    "#
                )
                .bind(user_id)
                .bind(card_master.id)
                .bind(cp)
                .bind(rarity)
                .bind(now)
                .bind(is_unique)
                .bind(unique_id)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    /// Record draw history
    async fn record_draw_history(
        &self,
        tx: &mut DatabaseTransaction,
        user_id: i32,
        card_id: i32,
        rarity: &CardRarity,
        cp: i32,
        is_paid: bool,
    ) -> AppResult<DrawHistory> {
        let now = Utc::now();

        match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, DrawHistory>(
                    r#"
                    INSERT INTO draw_history (user_id, card_id, status, cp, is_paid, drawn_at)
                    VALUES ($1, $2, $3, $4, $5, $6)
                    RETURNING *
                    "#
                )
                .bind(user_id)
                .bind(card_id)
                .bind(rarity)
                .bind(cp)
                .bind(is_paid)
                .bind(now)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, DrawHistory>(
                    r#"
                    INSERT INTO draw_history (user_id, card_id, status, cp, is_paid, drawn_at)
                    VALUES (?, ?, ?, ?, ?, ?)
                    RETURNING *
                    "#
                )
                .bind(user_id)
                .bind(card_id)
                .bind(rarity)
                .bind(cp)
                .bind(is_paid)
                .bind(now)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    /// Register unique card
    async fn register_unique_card(
        &self,
        tx: &mut DatabaseTransaction,
        user_card: &UserCard,
        owner_did: &str,
    ) -> AppResult<UniqueCardRegistry> {
        let unique_id = user_card.unique_id.ok_or_else(|| {
            AppError::Internal("Unique card must have unique_id".to_string())
        })?;

        match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, UniqueCardRegistry>(
                    r#"
                    INSERT INTO unique_card_registry (unique_id, card_id, owner_did, obtained_at)
                    VALUES ($1, $2, $3, $4)
                    RETURNING *
                    "#
                )
                .bind(unique_id)
                .bind(user_card.card_id)
                .bind(owner_did)
                .bind(user_card.obtained_at)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, UniqueCardRegistry>(
                    r#"
                    INSERT INTO unique_card_registry (unique_id, card_id, owner_did, obtained_at)
                    VALUES (?, ?, ?, ?)
                    RETURNING *
                    "#
                )
                .bind(unique_id)
                .bind(user_card.card_id)
                .bind(owner_did)
                .bind(user_card.obtained_at)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    /// Get or create user by DID
    async fn get_or_create_user(
        &self,
        tx: &mut DatabaseTransaction,
        did: &str,
    ) -> AppResult<User> {
        // Try to get existing user
        let existing_user = match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = $1")
                    .bind(did)
                    .fetch_optional(&mut **tx)
                    .await
                    .map_err(AppError::Database)?
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = ?")
                    .bind(did)
                    .fetch_optional(&mut **tx)
                    .await
                    .map_err(AppError::Database)?
            }
        };

        if let Some(user) = existing_user {
            return Ok(user);
        }

        // Create new user
        let handle = did.split('.').next().unwrap_or("unknown").to_string();
        let now = Utc::now();

        match tx {
            DatabaseTransaction::Postgres(tx) => {
                sqlx::query_as::<_, User>(
                    "INSERT INTO users (did, handle, created_at, updated_at) VALUES ($1, $2, $3, $4) RETURNING *"
                )
                .bind(did)
                .bind(&handle)
                .bind(now)
                .bind(now)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
            DatabaseTransaction::Sqlite(tx) => {
                sqlx::query_as::<_, User>(
                    "INSERT INTO users (did, handle, created_at, updated_at) VALUES (?, ?, ?, ?) RETURNING *"
                )
                .bind(did)
                .bind(&handle)
                .bind(now)
                .bind(now)
                .fetch_one(&mut **tx)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    /// Draw card with fallback rarity (when unique is unavailable)
    async fn draw_card_with_fallback(
        &self,
        tx: &mut DatabaseTransaction,
        user_id: i32,
        card_master: &CardMaster,
        fallback_rarity: CardRarity,
        is_paid: bool,
    ) -> AppResult<CardDrawResponse> {
        let cp = self.calculate_cp(card_master, &fallback_rarity);

        let user_card = self.create_user_card(
            tx,
            user_id,
            card_master,
            cp,
            &fallback_rarity,
            false,
        ).await?;

        let draw_history = self.record_draw_history(
            tx,
            user_id,
            card_master.id,
            &fallback_rarity,
            cp,
            is_paid,
        ).await?;

        Ok(CardDrawResponse {
            card: UserCardResponse {
                id: user_card.id,
                card_id: user_card.card_id,
                cp: user_card.cp,
                status: user_card.status,
                skill: user_card.skill,
                obtained_at: user_card.obtained_at,
                is_unique: user_card.is_unique,
                unique_id: user_card.unique_id,
            },
            master: CardMasterResponse {
                id: card_master.id,
                name: card_master.name.clone(),
                base_cp_min: card_master.base_cp_min,
                base_cp_max: card_master.base_cp_max,
                color: card_master.color.clone(),
                description: card_master.description.clone(),
            },
            is_unique: false,
            animation_type: self.get_animation_type(&fallback_rarity),
            draw_history_id: draw_history.id,
        })
    }

    /// Get animation type based on rarity
    fn get_animation_type(&self, rarity: &CardRarity) -> String {
        match rarity {
            CardRarity::Normal => "normal".to_string(),
            CardRarity::Rare => "sparkle".to_string(),
            CardRarity::SuperRare => "glow".to_string(),
            CardRarity::Kira => "rainbow".to_string(),
            CardRarity::Unique => "legendary".to_string(),
        }
    }

    /// Get gacha statistics
    pub async fn get_gacha_stats(&self, db: &Database) -> AppResult<GachaStatsResponse> {
        // Get total draws
        let total_draws: (i64,) = match db {
            Database::Postgres(pool) => {
                sqlx::query_as("SELECT COUNT(*) FROM draw_history")
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)?
            }
            Database::Sqlite(pool) => {
                sqlx::query_as("SELECT COUNT(*) FROM draw_history")
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)?
            }
        };

        // Get unique card counts
        let unique_counts: (i64, i64) = match db {
            Database::Postgres(pool) => {
                sqlx::query_as(
                    r#"
                    SELECT 
                        COUNT(*) as total,
                        (SELECT COUNT(*) FROM card_master) - COUNT(*) as available
                    FROM unique_card_registry
                    "#
                )
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)?
            }
            Database::Sqlite(pool) => {
                sqlx::query_as(
                    r#"
                    SELECT 
                        COUNT(*) as total,
                        (SELECT COUNT(*) FROM card_master) - COUNT(*) as available
                    FROM unique_card_registry
                    "#
                )
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)?
            }
        };

        // Get rarity distribution
        let rarity_breakdown = RarityBreakdown {
            normal: 0, // Would need actual counts from database
            rare: 0,
            super_rare: 0,
            kira: 0,
            unique: unique_counts.0 as i32,
        };

        Ok(GachaStatsResponse {
            probabilities: GachaProbabilities {
                normal: self.config.prob_normal,
                rare: self.config.prob_rare,
                super_rare: self.config.prob_super_rare,
                kira: self.config.prob_kira,
                unique: self.config.prob_unique,
                paid_multiplier: 2.0,
            },
            total_draws: total_draws.0 as i32,
            total_unique_cards: unique_counts.0 as i32,
            available_unique_cards: unique_counts.1 as i32,
            rarity_distribution: rarity_breakdown,
        })
    }
}