use crate::{
    database::Database,
    error::{AppError, AppResult},
    models::*,
};
use chrono::Utc;

pub struct UserService;

impl UserService {
    pub async fn get_user_by_did(db: &Database, did: &str) -> AppResult<Option<User>> {
        match db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = $1")
                    .bind(did)
                    .fetch_optional(pool)
                    .await
                    .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, User>("SELECT * FROM users WHERE did = ?")
                    .bind(did)
                    .fetch_optional(pool)
                    .await
                    .map_err(AppError::Database)
            }
        }
    }

    pub async fn create_user(db: &Database, did: &str, handle: &str) -> AppResult<User> {
        let now = Utc::now();

        match db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, User>(
                    "INSERT INTO users (did, handle, created_at, updated_at) VALUES ($1, $2, $3, $4) RETURNING *"
                )
                .bind(did)
                .bind(handle)
                .bind(now)
                .bind(now)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, User>(
                    "INSERT INTO users (did, handle, created_at, updated_at) VALUES (?, ?, ?, ?) RETURNING *"
                )
                .bind(did)
                .bind(handle)
                .bind(now)
                .bind(now)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    pub async fn update_user_handle(db: &Database, did: &str, handle: &str) -> AppResult<User> {
        let now = Utc::now();

        match db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, User>(
                    "UPDATE users SET handle = $1, updated_at = $2 WHERE did = $3 RETURNING *"
                )
                .bind(handle)
                .bind(now)
                .bind(did)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, User>(
                    "UPDATE users SET handle = ?, updated_at = ? WHERE did = ? RETURNING *"
                )
                .bind(handle)
                .bind(now)
                .bind(did)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)
            }
        }
    }

    pub async fn get_user_card_count(db: &Database, user_did: &str) -> AppResult<i64> {
        match db {
            Database::Postgres(pool) => {
                let row: (i64,) = sqlx::query_as("SELECT COUNT(*) FROM user_cards WHERE user_did = $1")
                    .bind(user_did)
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)?;
                Ok(row.0)
            }
            Database::Sqlite(pool) => {
                let row: (i64,) = sqlx::query_as("SELECT COUNT(*) FROM user_cards WHERE user_did = ?")
                    .bind(user_did)
                    .fetch_one(pool)
                    .await
                    .map_err(AppError::Database)?;
                Ok(row.0)
            }
        }
    }

    pub async fn get_user_unique_card_count(db: &Database, user_did: &str) -> AppResult<i64> {
        match db {
            Database::Postgres(pool) => {
                let row: (i64,) = sqlx::query_as(
                    "SELECT COUNT(*) FROM user_cards WHERE user_did = $1 AND is_unique = true"
                )
                .bind(user_did)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)?;
                Ok(row.0)
            }
            Database::Sqlite(pool) => {
                let row: (i64,) = sqlx::query_as(
                    "SELECT COUNT(*) FROM user_cards WHERE user_did = ? AND is_unique = 1"
                )
                .bind(user_did)
                .fetch_one(pool)
                .await
                .map_err(AppError::Database)?;
                Ok(row.0)
            }
        }
    }

    pub async fn get_user_cards_by_rarity(
        db: &Database,
        user_did: &str,
        rarity: CardRarity,
    ) -> AppResult<Vec<UserCardWithMasterQuery>> {
        match db {
            Database::Postgres(pool) => {
                sqlx::query_as::<_, UserCardWithMasterQuery>(
                    r#"
                    SELECT 
                        uc.id, uc.user_did, uc.card_id, uc.cp, uc.status, 
                        uc.obtained_at, uc.is_unique, uc.unique_id,
                        cm.id as master_id, cm.name, cm.base_cp_min, cm.base_cp_max, 
                        cm.color, cm.description
                    FROM user_cards uc
                    JOIN card_master cm ON uc.card_id = cm.id
                    WHERE uc.user_did = $1 AND uc.status = $2
                    ORDER BY uc.obtained_at DESC
                    "#
                )
                .bind(user_did)
                .bind(rarity.as_str())
                .fetch_all(pool)
                .await
                .map_err(AppError::Database)
            }
            Database::Sqlite(pool) => {
                sqlx::query_as::<_, UserCardWithMasterQuery>(
                    r#"
                    SELECT 
                        uc.id, uc.user_did, uc.card_id, uc.cp, uc.status, 
                        uc.obtained_at, uc.is_unique, uc.unique_id,
                        cm.id as master_id, cm.name, cm.base_cp_min, cm.base_cp_max, 
                        cm.color, cm.description
                    FROM user_cards uc
                    JOIN card_master cm ON uc.card_id = cm.id
                    WHERE uc.user_did = ? AND uc.status = ?
                    ORDER BY uc.obtained_at DESC
                    "#
                )
                .bind(user_did)
                .bind(rarity.as_str())
                .fetch_all(pool)
                .await
                .map_err(AppError::Database)
            }
        }
    }
}