import Foundation
import Combine
import SwiftUI

class CardManager: ObservableObject {
    @Published var userCards: [Card] = []
    @Published var isLoading = false
    @Published var errorMessage: String?
    @Published var currentDraw: CardDrawResult?
    @Published var isDrawing = false
    @Published var drawStatus: DrawStatus?
    @Published var canDraw: Bool = true
    
    private var cancellables = Set<AnyCancellable>()
    private let apiClient = APIClient.shared
    
    func loadUserCards(userDid: String) {
        isLoading = true
        
        // カードデータと制限状況を並行して取得
        let cardsPublisher = apiClient.getUserCards(userDid: userDid)
        let statusPublisher = apiClient.getDrawStatus(userDid: userDid)
        
        Publishers.CombineLatest(cardsPublisher, statusPublisher)
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] completion in
                    self?.isLoading = false
                    if case .failure(let error) = completion {
                        self?.errorMessage = self?.getErrorMessage(from: error)
                    }
                },
                receiveValue: { [weak self] cards, status in
                    self?.userCards = cards
                    self?.drawStatus = status
                    self?.canDraw = status.canDraw
                }
            )
            .store(in: &cancellables)
    }
    
    func checkDrawStatus(userDid: String) {
        apiClient.getDrawStatus(userDid: userDid)
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] completion in
                    if case .failure(let error) = completion {
                        self?.errorMessage = self?.getErrorMessage(from: error)
                    }
                },
                receiveValue: { [weak self] status in
                    self?.drawStatus = status
                    self?.canDraw = status.canDraw
                }
            )
            .store(in: &cancellables)
    }
    
    func drawCard(userDid: String, isPaid: Bool = false) {
        // 制限チェック
        guard canDraw else {
            if let nextTime = drawStatus?.nextDrawTime {
                let formatter = DateFormatter()
                formatter.dateStyle = .medium
                formatter.timeStyle = .short
                formatter.locale = Locale(identifier: "ja_JP")
                errorMessage = "カードを引けるのは\(formatter.string(from: nextTime))以降です。"
            } else {
                errorMessage = "現在カードを引くことができません。"
            }
            return
        }
        
        isDrawing = true
        errorMessage = nil
        
        apiClient.drawCard(userDid: userDid, isPaid: isPaid)
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] completion in
                    if case .failure(let error) = completion {
                        self?.isDrawing = false
                        self?.errorMessage = self?.getErrorMessage(from: error)
                        
                        // HTTP 429 (Too Many Requests) エラーの場合は制限状況を更新
                        if case .networkError(let message) = error, message.contains("429") {
                            self?.checkDrawStatus(userDid: userDid)
                        }
                    }
                },
                receiveValue: { [weak self] result in
                    self?.currentDraw = result
                    // カード引取後は制限状況を更新
                    self?.checkDrawStatus(userDid: userDid)
                    // アニメーション終了後にカードを追加
                }
            )
            .store(in: &cancellables)
    }
    
    func completeCardDraw() {
        if let newCard = currentDraw?.card {
            userCards.append(newCard)
        }
        currentDraw = nil
        isDrawing = false
    }
    
    private func getErrorMessage(from error: APIError) -> String {
        switch error {
        case .unauthorized:
            return "認証が必要です"
        case .networkError:
            return "ネットワークエラーが発生しました"
        default:
            return "エラーが発生しました"
        }
    }
}