import SwiftUI

struct CardView: View {
    let card: Card
    let isRevealing: Bool
    @State private var isFlipped = false
    
    init(card: Card, isRevealing: Bool = false) {
        self.card = card
        self.isRevealing = isRevealing
    }
    
    var body: some View {
        ZStack {
            // Card background
            RoundedRectangle(cornerRadius: 16)
                .fill(
                    LinearGradient(
                        gradient: Gradient(colors: gradientColors),
                        startPoint: .topLeading,
                        endPoint: .bottomTrailing
                    )
                )
                .frame(width: 200, height: 280)
            
            // Card content
            VStack(spacing: 16) {
                // Header
                HStack {
                    Text("#\(card.id)")
                        .font(.caption)
                        .foregroundColor(.white.opacity(0.7))
                    
                    Spacer()
                    
                    Text("CP: \(card.cp)")
                        .font(.caption)
                        .foregroundColor(.white.opacity(0.7))
                }
                
                Spacer()
                
                // Card name and icon
                VStack(spacing: 12) {
                    // Card icon (could be an image)
                    Circle()
                        .fill(Color(hex: cardInfo.color))
                        .frame(width: 60, height: 60)
                        .overlay(
                            Text(cardInfo.name.prefix(1))
                                .font(.title)
                                .fontWeight(.bold)
                                .foregroundColor(.white)
                        )
                    
                    Text(cardInfo.name)
                        .font(.title2)
                        .fontWeight(.bold)
                        .foregroundColor(.white)
                        .multilineTextAlignment(.center)
                    
                    if card.isUnique {
                        UniqueBadge()
                    }
                }
                
                Spacer()
                
                // Skill
                if let skill = card.skill, !skill.isEmpty {
                    Text(skill)
                        .font(.caption)
                        .foregroundColor(.white.opacity(0.8))
                        .padding(.horizontal, 8)
                        .padding(.vertical, 4)
                        .background(Color.black.opacity(0.3))
                        .cornerRadius(8)
                }
                
                // Rarity
                Text(card.status.displayName)
                    .font(.caption)
                    .foregroundColor(.white.opacity(0.7))
                    .textCase(.uppercase)
                    .tracking(1)
            }
            .padding(20)
            
            // Special effects
            if card.status == .kira {
                KiraEffect()
            } else if card.status == .unique {
                UniqueEffect()
            }
        }
        .rotation3DEffect(
            .degrees(isRevealing && !isFlipped ? 180 : 0),
            axis: (x: 0, y: 1, z: 0)
        )
        .onAppear {
            if isRevealing {
                DispatchQueue.main.asyncAfter(deadline: .now() + 0.5) {
                    withAnimation(.easeInOut(duration: 0.8)) {
                        isFlipped = true
                    }
                }
            }
        }
        .scaleEffect(isRevealing ? 1.1 : 1.0)
        .shadow(color: .black.opacity(0.3), radius: 10, x: 0, y: 5)
    }
    
    private var cardInfo: CardInfo {
        CardInfo.all[card.id] ?? CardInfo(id: card.id, name: "Unknown", color: "666666", description: "")
    }
    
    private var gradientColors: [Color] {
        card.status.gradientColors.map { Color(hex: $0) }
    }
}

struct UniqueBadge: View {
    @State private var phase: CGFloat = 0
    
    var body: some View {
        Text("UNIQUE")
            .font(.caption2)
            .fontWeight(.bold)
            .foregroundColor(.white)
            .padding(.horizontal, 12)
            .padding(.vertical, 4)
            .background(
                LinearGradient(
                    gradient: Gradient(colors: [
                        Color(hex: "ff00ff"),
                        Color(hex: "00ffff")
                    ]),
                    startPoint: .leading,
                    endPoint: .trailing
                )
                .hueRotation(.degrees(phase))
            )
            .cornerRadius(12)
            .onAppear {
                withAnimation(.linear(duration: 2).repeatForever(autoreverses: false)) {
                    phase = 360
                }
            }
    }
}

struct KiraEffect: View {
    @State private var sparkles: [SparkleData] = []
    
    var body: some View {
        ZStack {
            ForEach(sparkles, id: \.id) { sparkle in
                Image(systemName: "sparkle")
                    .foregroundColor(.yellow)
                    .font(.system(size: sparkle.size))
                    .position(x: sparkle.x, y: sparkle.y)
                    .opacity(sparkle.opacity)
            }
        }
        .onAppear {
            generateSparkles()
        }
    }
    
    private func generateSparkles() {
        for i in 0..<10 {
            let sparkle = SparkleData(
                id: i,
                x: CGFloat.random(in: 20...180),
                y: CGFloat.random(in: 20...260),
                size: CGFloat.random(in: 8...16),
                opacity: Double.random(in: 0.3...0.8)
            )
            sparkles.append(sparkle)
            
            DispatchQueue.main.asyncAfter(deadline: .now() + Double.random(in: 0...2)) {
                withAnimation(.easeInOut(duration: 1).repeatForever(autoreverses: true)) {
                    if let index = sparkles.firstIndex(where: { $0.id == sparkle.id }) {
                        sparkles[index].opacity = sparkles[index].opacity > 0.5 ? 0.2 : 0.8
                    }
                }
            }
        }
    }
}

struct UniqueEffect: View {
    @State private var pulseScale: CGFloat = 1.0
    
    var body: some View {
        RoundedRectangle(cornerRadius: 16)
            .stroke(
                LinearGradient(
                    gradient: Gradient(colors: [
                        Color(hex: "ff00ff"),
                        Color(hex: "00ffff"),
                        Color(hex: "ff00ff")
                    ]),
                    startPoint: .topLeading,
                    endPoint: .bottomTrailing
                ),
                lineWidth: 3
            )
            .scaleEffect(pulseScale)
            .opacity(0.8)
            .onAppear {
                withAnimation(.easeInOut(duration: 1.5).repeatForever(autoreverses: true)) {
                    pulseScale = 1.05
                }
            }
    }
}

struct SparkleData {
    let id: Int
    let x: CGFloat
    let y: CGFloat
    let size: CGFloat
    var opacity: Double
}

struct CardView_Previews: PreviewProvider {
    static var previews: some View {
        let sampleCard = Card(
            id: 0,
            cp: 100,
            status: .unique,
            skill: "サンプルスキル",
            ownerDid: "did:plc:example",
            obtainedAt: Date(),
            isUnique: true,
            uniqueId: "unique-123"
        )
        
        VStack {
            CardView(card: sampleCard)
            CardView(card: sampleCard, isRevealing: true)
        }
        .padding()
        .background(Color.black)
    }
}