import React, { useState, useEffect } from 'react';
import { Card } from './components/Card';
import { GachaAnimation } from './components/GachaAnimation';
import { Login } from './components/Login';
import { OAuthCallback } from './components/OAuthCallback';
import { CollectionAnalysis } from './components/CollectionAnalysis';
import { GachaStats } from './components/GachaStats';
import { CardBox } from './components/CardBox';
import { cardApi, aiCardApi } from './services/api';
import { authService, User } from './services/auth';
import { atprotoOAuthService } from './services/atproto-oauth';
import { Card as CardType, CardDrawResult } from './types/card';
import './App.css';

function App() {
  console.log('APP COMPONENT LOADED - Console working!');
  console.log('Current timestamp:', new Date().toISOString());
  
  // Immediately log URL information on every page load
  console.log('IMMEDIATE URL CHECK:');
  console.log('- href:', window.location.href);
  console.log('- pathname:', window.location.pathname); 
  console.log('- search:', window.location.search);
  console.log('- hash:', window.location.hash);
  
  // Also show URL info via alert if it contains OAuth parameters
  if (window.location.search.includes('code=') || window.location.search.includes('state=')) {
    const urlInfo = `OAuth callback detected!\n\nURL: ${window.location.href}\nSearch: ${window.location.search}`;
    alert(urlInfo);
    console.log('OAuth callback URL detected!');
  } else {
    // Check if we have stored OAuth info from previous steps
    const preOAuthUrl = sessionStorage.getItem('pre_oauth_url');
    const storedState = sessionStorage.getItem('oauth_state');
    const storedCodeVerifier = sessionStorage.getItem('oauth_code_verifier');
    
    console.log('=== OAUTH SESSION STORAGE CHECK ===');
    console.log('Pre-OAuth URL:', preOAuthUrl);
    console.log('Stored state:', storedState);
    console.log('Stored code verifier:', storedCodeVerifier ? 'Present' : 'Missing');
    console.log('=== END SESSION STORAGE CHECK ===');
  }
  
  const [isDrawing, setIsDrawing] = useState(false);
  const [currentDraw, setCurrentDraw] = useState<CardDrawResult | null>(null);
  const [userCards, setUserCards] = useState<CardType[]>([]);
  const [error, setError] = useState<string | null>(null);
  const [user, setUser] = useState<User | null>(null);
  const [showLogin, setShowLogin] = useState(false);
  const [isLoading, setIsLoading] = useState(true);
  const [activeTab, setActiveTab] = useState<'gacha' | 'collection' | 'analysis' | 'stats' | 'box'>('gacha');
  const [aiAvailable, setAiAvailable] = useState(false);

  useEffect(() => {
    // Handle popstate events for mock OAuth flow
    const handlePopState = () => {
      const urlParams = new URLSearchParams(window.location.search);
      const isOAuthCallback = urlParams.has('code') && urlParams.has('state');
      
      if (isOAuthCallback) {
        // Force re-render to handle OAuth callback
        window.location.reload();
      }
    };

    window.addEventListener('popstate', handlePopState);

    // Check if this is an OAuth callback
    const urlParams = new URLSearchParams(window.location.search);
    const isOAuthCallback = urlParams.has('code') && urlParams.has('state');
    
    if (isOAuthCallback) {
      return; // Let OAuthCallback component handle this
    }

    // Check existing sessions and AI availability
    const checkAuth = async () => {
      // Check AI availability
      const aiStatus = await aiCardApi.isAIAvailable();
      setAiAvailable(aiStatus);

      // First check OAuth session using official BrowserOAuthClient
      console.log('Checking OAuth session...');
      const oauthResult = await atprotoOAuthService.checkSession();
      console.log('OAuth checkSession result:', oauthResult);
      
      if (oauthResult) {
        console.log('OAuth session found:', oauthResult);
        // Ensure handle is not DID
        const handle = oauthResult.handle !== oauthResult.did ? oauthResult.handle : oauthResult.handle;
        setUser({ did: oauthResult.did, handle: handle });
        loadUserCards(oauthResult.did);
        setIsLoading(false);
        return;
      } else {
        console.log('No OAuth session found');
      }

      // Fallback to legacy auth
      const verifiedUser = await authService.verify();
      if (verifiedUser) {
        setUser(verifiedUser);
        loadUserCards(verifiedUser.did);
      }
      setIsLoading(false);
    };

    checkAuth();

    return () => {
      window.removeEventListener('popstate', handlePopState);
    };
  }, []);

  const loadUserCards = async (did: string) => {
    // Skip if DID is not resolved
    if (did === 'PENDING_DID_RESOLUTION') {
      console.log('Skipping card load for pending DID resolution');
      return;
    }
    
    try {
      console.log('Loading cards for DID:', did);
      const cards = await cardApi.getUserCards(did);
      console.log('Loaded cards:', cards);
      setUserCards(cards);
    } catch (err) {
      console.error('Failed to load cards:', err);
      console.error('DID used for request:', did);
      // ai.cardサーバーが起動していない場合の案内
      setError('カード取得に失敗しました。ai.cardサーバーが起動していることを確認してください。');
    }
  };

  const handleLogin = async (did: string, handle: string) => {
    // PENDING_DID_RESOLUTIONの場合はカード取得をスキップ
    if (did === 'PENDING_DID_RESOLUTION') {
      console.log('DID resolution pending, skipping card fetch');
      return;
    }
    
    setUser({ did, handle });
    setShowLogin(false);
    
    // 新規ユーザーの場合、初回ガチャでアカウント作成を促す
    try {
      const cards = await cardApi.getUserCards(did);
      setUserCards(cards);
    } catch (err: any) {
      // ユーザーが見つからない場合は、初回ガチャでアカウント作成
      if (err.message?.includes('User not found')) {
        console.log('新規ユーザーです。初回ガチャでアカウントが作成されます。');
        setUserCards([]);
      } else {
        console.error('Failed to load cards:', err);
      }
    }
  };

  const handleBackupCards = async () => {
    if (!user || userCards.length === 0) {
      alert('バックアップするカードがありません');
      return;
    }

    // デバッグ情報を表示
    const session = atprotoOAuthService.getSession();
    console.log('Current session:', session);
    console.log('User:', user);
    console.log('Cards to backup:', userCards);

    try {
      await atprotoOAuthService.saveCardToBox(userCards);
      alert(`${userCards.length}枚のカードをai.card.boxにバックアップしました！`);
    } catch (error) {
      console.error('バックアップエラー詳細:', error);
      
      const errorMessage = error instanceof Error ? error.message : 'Unknown error';
      
      // 認証エラーの場合は再ログインを促す
      if (errorMessage.includes('認証トークンが無効') || errorMessage.includes('InvalidToken')) {
        const shouldRelogin = confirm('認証トークンが無効です。再ログインしますか？');
        if (shouldRelogin) {
          handleLogout();
          setShowLogin(true);
        }
        return;
      }
      
      // その他のエラー
      const sessionInfo = session ? `DID: ${session.did}, Token: ${session.accessJwt?.substring(0, 20)}...` : 'No session';
      alert(`バックアップに失敗しました。\n\nエラー: ${errorMessage}\nセッション: ${sessionInfo}\n\n詳細はコンソールを確認してください。`);
    }
  };

  const checkTokenStatus = () => {
    // Get session from service
    const session = atprotoOAuthService.getSession();
    console.log('checkTokenStatus - session:', session);
    
    // Also check the agent directly
    const agent = atprotoOAuthService.getAgent();
    console.log('checkTokenStatus - agent:', agent);
    console.log('checkTokenStatus - agent.session:', agent?.session);
    
    if (session) {
      const tokenInfo = `
DID: ${session.did}
Handle: ${session.handle}
Access Token: ${session.accessJwt?.substring(0, 30)}...
Refresh Token: ${session.refreshJwt?.substring(0, 30)}...
      `.trim();
      alert(`認証状態:\n\n${tokenInfo}`);
    } else if (agent?.session) {
      // If getSession failed but agent has session, use that
      const tokenInfo = `
DID: ${agent.session.did}
Handle: ${agent.session.handle || 'unknown'}
Access Token: ${agent.session.accessJwt?.substring(0, 30) || 'N/A'}...
Refresh Token: ${agent.session.refreshJwt?.substring(0, 30) || 'N/A'}...
      `.trim();
      alert(`認証状態(Agent):\n\n${tokenInfo}`);
    } else {
      alert('セッションが見つかりません');
    }
  };

  const setManualTokens = () => {
    const accessJwt = prompt('Access JWT を入力してください (あなたのシェルスクリプトで取得したもの):');
    const refreshJwt = prompt('Refresh JWT を入力してください:');
    
    if (accessJwt && refreshJwt) {
      try {
        atprotoOAuthService.setManualTokens(accessJwt, refreshJwt);
        alert('トークンが設定されました！再ログインしてください。');
        window.location.reload();
      } catch (error) {
        alert('トークンの設定に失敗しました: ' + error);
      }
    }
  };

  const handleLogout = async () => {
    // Logout from both services
    await authService.logout();
    atprotoOAuthService.logout();
    setUser(null);
    setUserCards([]);
  };

  const handleDraw = async (isPaid: boolean = false) => {
    if (!user || user.did === 'PENDING_DID_RESOLUTION') {
      setShowLogin(true);
      return;
    }

    setIsDrawing(true);
    setError(null);

    try {
      const result = await cardApi.drawCard(user.did, isPaid);
      setCurrentDraw(result);
    } catch (err) {
      setError('カードの抽選に失敗しました');
      setIsDrawing(false);
    }
  };

  const handleAnimationComplete = () => {
    if (currentDraw) {
      setUserCards([...userCards, currentDraw.card]);
      setCurrentDraw(null);
      setIsDrawing(false);
    }
  };

  // OAuth callback is now handled by React Router in main.tsx
  console.log('=== APP.TSX URL CHECK ===');
  console.log('Full URL:', window.location.href);
  console.log('Pathname:', window.location.pathname);
  console.log('Search params:', window.location.search);
  console.log('=== END URL CHECK ===');

  if (isLoading) {
    return (
      <div className="app">
        <div className="loading">Loading...</div>
      </div>
    );
  }

  return (
    <div className="app">
      <header className="app-header">
        <h1>ai.card</h1>
        <p>atprotoベースカードゲーム</p>
        <div className="user-info">
          {user ? (
            <>
              <span className="user-handle">@{user.handle}</span>
              <button onClick={handleBackupCards} className="backup-button">
                💾 カードバックアップ
              </button>
              <button onClick={checkTokenStatus} className="token-button">
                🔑 認証状態
              </button>
              <button onClick={setManualTokens} className="token-button">
                🔧 トークン設定
              </button>
              <button onClick={handleLogout} className="logout-button">
                ログアウト
              </button>
            </>
          ) : (
            <button onClick={() => setShowLogin(true)} className="login-button">
              ログイン
            </button>
          )}
        </div>
      </header>

      <nav className="app-nav">
        <button 
          className={`nav-button ${activeTab === 'gacha' ? 'active' : ''}`}
          onClick={() => setActiveTab('gacha')}
        >
          🎲 ガチャ
        </button>
        <button 
          className={`nav-button ${activeTab === 'collection' ? 'active' : ''}`}
          onClick={() => setActiveTab('collection')}
        >
          📚 コレクション
        </button>
        {user && (
          <>
            {aiAvailable && (
              <button 
                className={`nav-button ${activeTab === 'analysis' ? 'active' : ''}`}
                onClick={() => setActiveTab('analysis')}
              >
                🧠 AI分析
              </button>
            )}
            <button 
              className={`nav-button ${activeTab === 'stats' ? 'active' : ''}`}
              onClick={() => setActiveTab('stats')}
            >
              📊 統計 {aiAvailable ? '(AI強化)' : ''}
            </button>
            <button 
              className={`nav-button ${activeTab === 'box' ? 'active' : ''}`}
              onClick={() => setActiveTab('box')}
            >
              📦 カードボックス
            </button>
          </>
        )}
      </nav>

      <main className="app-main">
        {activeTab === 'gacha' && (
          <section className="gacha-section">
            <h2>カードを引く</h2>
            <div className="gacha-buttons">
              <button 
                onClick={() => handleDraw(false)} 
                disabled={isDrawing}
                className="gacha-button"
              >
                通常ガチャ
              </button>
              <button 
                onClick={() => handleDraw(true)} 
                disabled={isDrawing}
                className="gacha-button gacha-button-premium"
              >
                プレミアムガチャ
              </button>
            </div>
            {error && <p className="error">{error}</p>}
          </section>
        )}

        {activeTab === 'collection' && (
          <section className="collection-section">
            <h2>コレクション</h2>
            <div className="card-grid">
              {userCards.map((card, index) => (
                <Card key={index} card={card} detailed={false} />
              ))}
            </div>
            {userCards.length === 0 && (
              <p className="empty-message">
                {user ? 'まだカードを持っていません' : 'ログインしてカードを集めよう'}
              </p>
            )}
          </section>
        )}

        {activeTab === 'analysis' && user && aiAvailable && (
          <section className="analysis-section">
            <h2>🧠 AI コレクション分析</h2>
            <CollectionAnalysis userDid={user.did} />
          </section>
        )}

        {activeTab === 'stats' && (
          <section className="stats-section">
            <h2>📊 ガチャ統計</h2>
            <GachaStats />
          </section>
        )}

        {activeTab === 'box' && user && (
          <section className="box-section">
            <h2>📦 atproto カードボックス</h2>
            <CardBox userDid={user.did} />
          </section>
        )}
      </main>

      {currentDraw && (
        <GachaAnimation
          card={currentDraw.card}
          animationType={currentDraw.animation_type}
          onComplete={handleAnimationComplete}
        />
      )}

      {showLogin && (
        <Login
          onLogin={handleLogin}
          onClose={() => setShowLogin(false)}
        />
      )}
    </div>
  );
}

export default App;